SystemOrganization addCategory: #'Zinc-SSO-OAuth2-Core'!

Object subclass: #ZnOAuth2ConsumerData
	instanceVariableNames: 'key secret redirectUrl'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnOAuth2ConsumerData commentStamp: '<historical>' prior: 0!
I am ZnOAuth2ConsumerData holding the data necessary to ask an OAuth2 provider to do authentication on our behalf.

key
	- the application key

redirectUrl
	- The url to which to provider will redirect to

secret
	- the application secret!

!ZnOAuth2ConsumerData class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 10/7/2012 16:04'!
key: aKey secret: aSecret

	^ self new setKey: aKey andSecret: aSecret! !

!ZnOAuth2ConsumerData class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 12/7/2012 15:40'!
key: aKey secret: aSecret redirectUrl: anUrl

	^ self new setKey: aKey andSecret: aSecret andRedirectUrl: anUrl! !

!ZnOAuth2ConsumerData methodsFor: 'accessing' stamp: 'JanVanDeSandt 10/7/2012 13:37'!
key
	^ key! !

!ZnOAuth2ConsumerData methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/7/2012 15:23'!
redirectUrl

	^ redirectUrl ! !

!ZnOAuth2ConsumerData methodsFor: 'accessing' stamp: 'JanVanDeSandt 10/7/2012 13:37'!
secret
	^ secret! !

!ZnOAuth2ConsumerData methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 10/7/2012 16:03'!
setKey: aKey andSecret: aSecret

	key := aKey.
	secret := aSecret ! !

!ZnOAuth2ConsumerData methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 12/7/2012 15:40'!
setKey: aKey andSecret: aSecret andRedirectUrl: anUrl

	self setKey: aKey andSecret: aSecret.
	redirectUrl := anUrl! !

Object subclass: #ZnOAuth2Session
	instanceVariableNames: 'consumerData redirectUrl scopes accessToken tokenType refreshToken'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnOAuth2Session commentStamp: '<historical>' prior: 0!
I am ZnOAuth2Session, representing server side state tracking an OAuth2 session.

consumerData  
	- needed to setup an authentication request with a provider
	
accessToken
	- the result of a successful authentication
	
scopes
	- collection of strings representing the permissions 	
	
redirectUrl
	- the URL needed by the provider to get back to us

tokenType
	- Indicates the type of token returned (not really used yet). 
	
refreshToken 
	- a token that may be used to obtain a new access token. This field is only present if access_type=offline is included in the authorization code request (currently only used by Google).!

ZnOAuth2Session subclass: #ZnAmazonOAuth2Session
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnAmazonOAuth2Session commentStamp: '<historical>' prior: 0!
I am ZnAmazonOAuth2Session.
I am a ZnOAuth2Session.!

!ZnAmazonOAuth2Session methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 6/2/2013 12:18'!
authenticationUrl

	^ (ZnUrl fromString: 'https://www.amazon.com/ap/oa')
		queryAt: 'client_id' put: self consumerData key ;
		queryAt: 'response_type' put: 'code' ;
		queryAt: 'redirect_uri' put: self redirectUrl ;
		queryAt: 'scope' put: self scopeParameter ;
		yourself.
! !

!ZnAmazonOAuth2Session methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 6/2/2013 12:22'!
handleAuthenticationCallback: aParameterDictionary 

	| code url oauthResponse |
	
	code := aParameterDictionary at: 'code' ifAbsent: [ ^nil ].

	url := (ZnUrl fromString: 'https://api.amazon.com/auth/o2/token')
		queryAt: 'grant_type' put: 'authorization_code' ;
		queryAt: 'client_id' put: self consumerData key ;
		queryAt: 'client_secret' put: self consumerData secret ;
		queryAt: 'code' put: code;
		yourself.
	
	oauthResponse := ZnClient new get: url.
	
	accessToken := (self fieldDictionaryFrom: oauthResponse) at: 'access_token'.! !

ZnOAuth2Session subclass: #ZnFacebookOAuth2Session
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnFacebookOAuth2Session commentStamp: '<historical>' prior: 0!
I am ZnFacebookOAuth2Session.
I am a ZnOAuth2Session.!

!ZnFacebookOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:10'!
scopeEmail
	"Provides access to the user's primary email address in the email property.
	See: https://developers.facebook.com/docs/reference/login/email-permissions/ "
	
	^ 'email'! !

!ZnFacebookOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:14'!
scopeUserLikes
	"Provides access to the list of all of the pages the user has liked as the likes connection
	See: https://developers.facebook.com/docs/reference/login/extended-profile-properties/"
	
	^ 'user_likes'! !

!ZnFacebookOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:16'!
scopeUserPhotos
	"Provides access to the photos the user has uploaded.
	See: https://developers.facebook.com/docs/reference/login/extended-profile-properties/"
	
	^ 'user_photos'! !

!ZnFacebookOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:26'!
ssoScopes
	"Answer a collection of scopes that can be used for Authentication. "

	^ Array with: self scopeEmail! !

!ZnFacebookOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/23/2012 23:34'!
authenticationUrl

	^ (ZnUrl fromString: 'https://graph.facebook.com/oauth/authorize')
		queryAt: 'client_id' put: self consumerData key ;
		queryAt: 'redirect_uri' put: self redirectUrl ;
		queryAt: 'scope' put: self scopeParameter ;
		yourself.
! !

!ZnFacebookOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 10/7/2012 14:31'!
getUserData

	| url response |
	
	url := (ZnUrl fromString: 'https://graph.facebook.com/me')
		queryAt: 'access_token' put: self accessToken;
		yourself.

	response := ZnClient new get: url.
	
	^ NeoJSONReader fromString: response! !

!ZnFacebookOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/7/2012 15:37'!
handleAuthenticationCallback: aParameterDictionary 

	| code url oauthResponse |
	
	code := aParameterDictionary at: 'code' ifAbsent: [ ^nil ].

	url := (ZnUrl fromString: 'https://graph.facebook.com/oauth/access_token')
		queryAt: 'client_id' put: self consumerData key ;
		queryAt: 'redirect_uri' put: self redirectUrl ;
		queryAt: 'client_secret' put: self consumerData secret ;
		queryAt: 'code' put: code;
		yourself.
	
	oauthResponse := ZnClient new get: url.
	
	accessToken := (self fieldDictionaryFrom: oauthResponse) at: 'access_token'.! !

ZnOAuth2Session subclass: #ZnGoogleOAuth2Session
	instanceVariableNames: 'accessType approvalPrompt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnGoogleOAuth2Session commentStamp: '<historical>' prior: 0!
I am ZnGoogleOAuth2Session.
I am a ZnOAuthSession.

Google OAuth2 docs: https://developers.google.com/accounts/docs/OAuth2WebServer.

accessType - hold the access type value described by the Google OAuth2 docs. The value can be: "online" or "offline".

approvalPrompt - hold the approval_prompt value described by the Google OAuth2 docs. The value can be: "auto" or "force". It is useful to use "force" when in "offline" mode, because a refresh_token is sent only once after user consent is obtained. If the refresh_token is lost, then the user must be re-prompted for consent (which won't happen with the default "auto" setting).
!

!ZnGoogleOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:01'!
scopeUserinfoEmail
	"Gain read-only access to the user's email address."

	^ 'https://www.googleapis.com/auth/userinfo.email'! !

!ZnGoogleOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:00'!
scopeUserinfoProfile
	"Gain read-only access to basic profile information, including a user identifier, name, profile photo, 
	profile URL, country, language, timezone, and birthdate. "

	^ 'https://www.googleapis.com/auth/userinfo.profile'! !

!ZnGoogleOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:25'!
ssoScopes
	"Answer a collection of scopes that can be used for Authentication. "

	^ Array with: self scopeUserinfoProfile with: self scopeUserinfoEmail! !

!ZnGoogleOAuth2Session methodsFor: 'accessing' stamp: 'YanniChiu 1/31/2013 19:32'!
accessType
	^ accessType! !

!ZnGoogleOAuth2Session methodsFor: 'accessing' stamp: 'YanniChiu 1/31/2013 19:32'!
accessType: anObject
	accessType := anObject! !

!ZnGoogleOAuth2Session methodsFor: 'accessing' stamp: 'YanniChiu 2/7/2013 10:17'!
approvalPrompt
	^ approvalPrompt! !

!ZnGoogleOAuth2Session methodsFor: 'accessing' stamp: 'YanniChiu 2/7/2013 10:17'!
approvalPrompt: anObject
	approvalPrompt := anObject! !

!ZnGoogleOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/7/2012 16:11'!
authenticationUrl

	^self authenticationUrlWithState: 'no-op'! !

!ZnGoogleOAuth2Session methodsFor: 'public' stamp: 'YanniChiu 2/7/2013 10:21'!
authenticationUrlWithState: aStateString

	| authUrl |

	authUrl := (ZnUrl fromString: 'https://accounts.google.com/o/oauth2/auth')
		queryAt: 'response_type' put: 'code' ;
		queryAt: 'client_id' put: self consumerData key ;
		queryAt: 'redirect_uri' put: self redirectUrl ;
		queryAt: 'scope' put: self scopeParameter ;
		queryAt: 'state' put: aStateString ;
		queryAt: 'access_type' put: self accessType ;
		yourself.

	self isForceApprovalPrompt
		ifTrue: [ authUrl queryAt: 'approval_prompt' put: 'force' ].
		
	^authUrl! !

!ZnGoogleOAuth2Session methodsFor: 'accessing' stamp: 'YanniChiu 1/31/2013 19:31'!
beOfflineAccessType
	self accessType: 'offline'! !

!ZnGoogleOAuth2Session methodsFor: 'accessing' stamp: 'YanniChiu 1/31/2013 19:32'!
beOnlineAccessType
	self accessType: 'online'! !

!ZnGoogleOAuth2Session methodsFor: 'accessing' stamp: 'YanniChiu 2/7/2013 10:18'!
forceApprovalPrompt
	self approvalPrompt: 'force'! !

!ZnGoogleOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 10/7/2012 15:27'!
getUserData

	| url response |
	
	url := (ZnUrl fromString: 'https://www.googleapis.com/oauth2/v1/userinfo')
		queryAt: 'access_token' put: self accessToken;
		yourself.

	response := ZnClient new get: url.
	
	^ NeoJSONReader fromString: response! !

!ZnGoogleOAuth2Session methodsFor: 'public' stamp: 'YanniChiu 2/7/2013 11:55'!
handleAuthenticationCallback: aParameterDictionary 

	| code tokenRequest tokenResponse tokenData |
	
	code := aParameterDictionary at: 'code' ifAbsent: [ ^nil ].

	tokenRequest  := (ZnRequest post: (ZnUrl fromString: 'https://accounts.google.com/o/oauth2/token'))
		entity: (ZnApplicationFormUrlEncodedEntity new
			at: 'code' put: code ;
			at: 'client_id' put: self consumerData key ;
			at: 'client_secret' put: self consumerData secret ;
			at: 'redirect_uri' put: self redirectUrl ;
			at: 'grant_type' put: 'authorization_code' ;
			yourself) ;
		yourself.

	tokenResponse := ZnClient new 
		request: tokenRequest ;
		post.
	tokenData := NeoJSONReader fromString: tokenResponse.
		
	accessToken := tokenData at: 'access_token'.
	tokenType := tokenData at: 'token_type'.
	(tokenData includesKey: 'refresh_token')
		ifTrue: [ refreshToken := tokenData at: 'refresh_token' ].
! !

!ZnGoogleOAuth2Session methodsFor: 'public' stamp: 'YanniChiu 2/7/2013 11:54'!
handleTokenRefresh
	| tokenRequest tokenResponse tokenData |
	
	tokenRequest  := (ZnRequest post: (ZnUrl fromString: 'https://accounts.google.com/o/oauth2/token'))
		entity: (ZnApplicationFormUrlEncodedEntity new
			at: 'client_id' put: self consumerData key ;
			at: 'client_secret' put: self consumerData secret ;
			at: 'grant_type' put: 'refresh_token' ;
			at: 'refresh_token' put: self refreshToken ;
			yourself) ;
		yourself.

	tokenResponse := ZnClient new 
		request: tokenRequest ;
		post.

	tokenData := NeoJSONReader fromString: tokenResponse.
	(tokenData includesKey: 'error')
		ifTrue: [ Error signal: (tokenData at: 'error') asString ].

	accessToken := tokenData at: 'access_token'.
	tokenType := tokenData at: 'token_type'.
! !

!ZnGoogleOAuth2Session methodsFor: 'initialize-release' stamp: 'YanniChiu 1/31/2013 19:34'!
initialize
	super initialize.
	self beOnlineAccessType.
! !

!ZnGoogleOAuth2Session methodsFor: 'testing' stamp: 'YanniChiu 2/7/2013 10:19'!
isForceApprovalPrompt
	^ self approvalPrompt notNil and: [ self approvalPrompt = 'force' ]! !

!ZnGoogleOAuth2Session methodsFor: 'testing' stamp: 'YanniChiu 2/5/2013 15:04'!
isOfflineAccessType
	^ self accessType = 'offline'! !

ZnOAuth2Session subclass: #ZnMicrosoftOAuth2Session
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnMicrosoftOAuth2Session commentStamp: '<historical>' prior: 0!
I am ZnMicrosoftOAuth2Session.
I am a ZnOAuth2Session.

Also see: http://msdn.microsoft.com/en-us/library/live/hh243647.aspx!

!ZnMicrosoftOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:18'!
scopeBasic
	"Read access to a user's basic profile info. Also enables read access to a user's list of contacts.
	See: http://msdn.microsoft.com/en-us/library/live/hh243646.aspx "

	^ 'wl.basic'! !

!ZnMicrosoftOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:21'!
scopeEmails
	"Read access to a user's personal, preferred, and business email addresses.
	See: http://msdn.microsoft.com/en-us/library/live/hh243646.aspx "

	^ 'wl.emails'! !

!ZnMicrosoftOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:20'!
scopeSignin
	"Single sign-in behavior. With single sign-in, users who are already signed in to Live Connect
	are also signed in to your website.
	See: http://msdn.microsoft.com/en-us/library/live/hh243646.aspx "

	^ 'wl.signin'! !

!ZnMicrosoftOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:25'!
ssoScopes
	"Answer a collection of scopes that can be used for Authentication. "

	^ Array with: self scopeSignin with: self scopeBasic with: self scopeEmails! !

!ZnMicrosoftOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/23/2012 23:36'!
authenticationUrl

	^ (ZnUrl fromString: 'https://login.live.com/oauth20_authorize.srf')
		queryAt: 'response_type' put: 'code' ;
		queryAt: 'client_id' put: self consumerData key ;
		queryAt: 'redirect_uri' put: self redirectUrl ;
		queryAt: 'scope' put: self scopeParameter ;
		yourself.
! !

!ZnMicrosoftOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/7/2012 21:06'!
getUserData

	| url response |
	
	url := (ZnUrl fromString: 'https://apis.live.net/v5.0/me')
		queryAt: 'access_token' put: self accessToken;
		yourself.

	response := ZnClient new get: url.
	
	^ NeoJSONReader fromString: response! !

!ZnMicrosoftOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/7/2012 21:13'!
handleAuthenticationCallback: aParameterDictionary 

	| code tokenRequest tokenResponse tokenData |
	
	code := aParameterDictionary at: 'code' ifAbsent: [ ^nil ].

	tokenRequest  := (ZnRequest post: (ZnUrl fromString: 'https://login.live.com/oauth20_token.srf'))
		entity: (ZnApplicationFormUrlEncodedEntity new
			at: 'code' put: code ;
			at: 'client_id' put: self consumerData key ;
			at: 'client_secret' put: self consumerData secret ;
			at: 'redirect_uri' put: self redirectUrl ;
			at: 'grant_type' put: 'authorization_code' ;
			yourself) ;
		yourself.

	tokenResponse := ZnClient new 
		request: tokenRequest ;
		post.
	tokenData := NeoJSONReader fromString: tokenResponse.
		
	accessToken := tokenData at: 'access_token'! !

!ZnOAuth2Session class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 6/2/2013 12:32'!
amazon: aConsumerData

	^ ZnAmazonOAuth2Session new
		setConsumerData: aConsumerData ;
		yourself! !

!ZnOAuth2Session class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 12/7/2012 15:32'!
facebook: aConsumerData

	^ ZnFacebookOAuth2Session new
		setConsumerData: aConsumerData ;
		yourself! !

!ZnOAuth2Session class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 12/7/2012 15:32'!
google: aConsumerData

	^ ZnGoogleOAuth2Session new
		setConsumerData: aConsumerData ;
		yourself! !

!ZnOAuth2Session class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 12/7/2012 20:57'!
microsoft: aConsumerData

	^ ZnMicrosoftOAuth2Session new
		setConsumerData: aConsumerData ;
		yourself! !

!ZnOAuth2Session class methodsFor: 'resources' stamp: 'JanVanDeSandt 12/23/2012 23:42'!
ssoScopes
	"Answer a collection of scopes that can be used for Authentication. "

	^ #()! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'JanVanDeSandt 10/7/2012 14:04'!
accessToken
	^ accessToken! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'YanniChiu 2/6/2013 14:21'!
accessToken: anObject
	accessToken := anObject! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'YanniChiu 1/31/2013 19:20'!
addScope: aScopeString
	self scopes add: aScopeString! !

!ZnOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 10/7/2012 15:32'!
authenticationUrl
	"Answer a URL to the login page of the Identity Provider"
	
	self subclassResponsibility 
	! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'JanVanDeSandt 10/7/2012 14:04'!
consumerData
	^ consumerData! !

!ZnOAuth2Session methodsFor: 'private' stamp: 'JanVanDeSandt 10/7/2012 14:17'!
fieldDictionaryFrom: aString

	| dict stream |
	
	dict := Dictionary new.
	stream := aString readStream.
	[ stream atEnd ] whileFalse: [ dict at: (stream upTo: $=) put: (stream upTo: $&) ].

	^dict! !

!ZnOAuth2Session methodsFor: 'public' stamp: 'JanVanDeSandt 12/7/2012 14:38'!
handleAuthenticationCallback: aParameterDictionary 

	self subclassResponsibility ! !

!ZnOAuth2Session methodsFor: 'initialize-release' stamp: 'YanniChiu 1/31/2013 19:17'!
initialize

	super initialize.
	
	scopes := OrderedCollection withAll: self class ssoScopes ! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/7/2012 15:34'!
redirectUrl
	^ redirectUrl! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/7/2012 15:34'!
redirectUrl: anObject
	redirectUrl := anObject! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'YanniChiu 2/5/2013 15:14'!
refreshToken
	^ refreshToken! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'YanniChiu 2/6/2013 14:21'!
refreshToken: anObject
	refreshToken := anObject! !

!ZnOAuth2Session methodsFor: 'private' stamp: 'JanVanDeSandt 12/23/2012 23:32'!
scopeParameter

	^ String streamContents: [ :stream |
		self scopes
			do: [ :each | stream nextPutAll: each ]
			separatedBy: [ stream nextPut: Character space ] ]! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/23/2012 23:32'!
scopes

	^ scopes ifNil: [ scopes := OrderedCollection new ]! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'JanVanDeSandt 12/23/2012 23:30'!
scopes: aCollection

	scopes := aCollection! !

!ZnOAuth2Session methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 6/2/2013 12:00'!
setConsumerData: aConsumerData

	consumerData := aConsumerData.
	redirectUrl := aConsumerData redirectUrl
! !

!ZnOAuth2Session methodsFor: 'accessing' stamp: 'YanniChiu 2/5/2013 15:26'!
tokenType
	^ tokenType! !

ZnOAuth2Session subclass: #ZnOpenIDConnectSession
	instanceVariableNames: 'discoveryDocument authenticationState extraAuthenticationParameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnOpenIDConnectSession commentStamp: 'JanVanDeSandt 3/21/2014 14:34' prior: 0!
I'm a ZnOpenIDConnectSession, representing server side state tracking an OpenID Connect session.

See: http://openid.net/connect/

Instance Variables
	discoveryDocument:		<ZnOpenIDConnectDiscoveryDocument>
	extraAuthenticationParameters:		<Dictionary>
	idToken:		<Object>
!

ZnOpenIDConnectSession subclass: #ZnGoogleOpenIDConnectSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnGoogleOpenIDConnectSession commentStamp: 'JanVanDeSandt 3/21/2014 14:33' prior: 0!
A ZnGoogleOpenIDConnectSession is used to support Google specific extensions of the OpenID Connect standard.

See https://developers.google.com/accounts/docs/OAuth2Login
!

!ZnGoogleOpenIDConnectSession class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 3/21/2014 14:05'!
withConsumerData: aConsumerData

	^self new
		initializeWith: ZnOpenIDConnectDiscoveryDocument google consumerData: aConsumerData ! !

!ZnGoogleOpenIDConnectSession methodsFor: 'accessing authentication parameters' stamp: 'JanVanDeSandt 3/21/2014 14:36'!
accessType

	^self extraAuthenticationParameterAt: 'access_type'! !

!ZnGoogleOpenIDConnectSession methodsFor: 'accessing authentication parameters' stamp: 'JanVanDeSandt 3/21/2014 14:36'!
accessType: aString

	self extraAuthenticationParameterAt: 'access_type' put: aString! !

!ZnGoogleOpenIDConnectSession methodsFor: 'accessing authentication parameters' stamp: 'JanVanDeSandt 3/21/2014 14:02'!
beOfflineAccessType

	self accessType: 'offline'! !

!ZnGoogleOpenIDConnectSession methodsFor: 'accessing authentication parameters' stamp: 'JanVanDeSandt 3/21/2014 14:02'!
beOnlineAccessType

	self accessType: 'online'! !

!ZnGoogleOpenIDConnectSession methodsFor: 'accessing authentication parameters' stamp: 'JanVanDeSandt 3/21/2014 14:36'!
hostedDomain

	^self extraAuthenticationParameterAt: 'hd'! !

!ZnGoogleOpenIDConnectSession methodsFor: 'accessing authentication parameters' stamp: 'JanVanDeSandt 3/21/2014 14:37'!
hostedDomain: aString

	self extraAuthenticationParameterAt: 'hd' put: aString.! !

!ZnGoogleOpenIDConnectSession methodsFor: 'accessing authentication parameters' stamp: 'JanVanDeSandt 3/21/2014 14:37'!
loginHint

	^self extraAuthenticationParameterAt: 'login_hint'! !

!ZnGoogleOpenIDConnectSession methodsFor: 'accessing authentication parameters' stamp: 'JanVanDeSandt 3/21/2014 14:37'!
loginHint: aString

	self extraAuthenticationParameterAt: 'login_hint' put: aString.! !

!ZnGoogleOpenIDConnectSession methodsFor: 'private' stamp: 'JanVanDeSandt 3/21/2014 14:43'!
validateIdToken: aString
	"TO DO!!"
	
"From Google: https://developers.google.com/accounts/docs/OAuth2Login#obtainuserinfo

An ID token is a cryptographically signed JSON object encoded in base 64. Normally, it is critical that you validate an ID token before you use it, but since you are communicating directly with Google over an intermediary-free HTTPS channel and using your client secret to authenticate yourself to Google, you can be confident that the token you receive really comes from Google and is valid. If your server passes the ID token to other components of your app, it is extremely important that the other components validate the token before using it."! !

!ZnOpenIDConnectSession class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/21/2014 13:07'!
ssoScopes

	^#( 'openid' 'profile' 'email' )! !

!ZnOpenIDConnectSession methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/21/2014 14:49'!
authenticationState

	^authenticationState! !

!ZnOpenIDConnectSession methodsFor: 'public' stamp: 'JanVanDeSandt 3/21/2014 14:59'!
authenticationUrl

	authenticationState := self secureRandomString.
	^self authenticationUrlWithState: authenticationState.! !

!ZnOpenIDConnectSession methodsFor: 'public' stamp: 'JanVanDeSandt 3/21/2014 14:52'!
authenticationUrlWithState: aStateString

	| authUrl |

	authUrl := (ZnUrl fromString: self authorizationEndpoint)
		queryAt: 'response_type' put: 'code' ;
		queryAt: 'client_id' put: self consumerData key ;
		queryAt: 'redirect_uri' put: self redirectUrl ;
		queryAt: 'scope' put: self scopeParameter ;
		queryAt: 'state' put: aStateString ;
		queryAddAll: self extraAuthenticationParameters ;
		yourself.

	^authUrl! !

!ZnOpenIDConnectSession methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/21/2014 13:10'!
authorizationEndpoint

	^discoveryDocument authorizationEndpoint! !

!ZnOpenIDConnectSession methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/21/2014 14:35'!
extraAuthenticationParameterAt: aKey

	^self extraAuthenticationParameters at: aKey ifAbsent: [ nil ]! !

!ZnOpenIDConnectSession methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/21/2014 14:36'!
extraAuthenticationParameterAt: aKey put: aStringOrNil

	aStringOrNil isNil
		ifTrue: [ self extraAuthenticationParameters removeKey: aKey ifAbsent: [  ] ]
		ifFalse: [ self extraAuthenticationParameters at: aKey put: aStringOrNil ]! !

!ZnOpenIDConnectSession methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/21/2014 13:50'!
extraAuthenticationParameters

	^extraAuthenticationParameters ifNil: [ extraAuthenticationParameters := Dictionary new ]! !

!ZnOpenIDConnectSession methodsFor: 'public' stamp: 'JanVanDeSandt 3/21/2014 13:34'!
getUserData

	| url response |
	
	url := (ZnUrl fromString: self userinfoEndpoint)
		queryAt: 'access_token' put: self accessToken;
		yourself.

	response := ZnClient new get: url.
	
	^ NeoJSONReader fromString: response! !

!ZnOpenIDConnectSession methodsFor: 'public' stamp: 'JanVanDeSandt 3/21/2014 14:51'!
handleAuthenticationCallback: aParameterDictionary 

	| code tokenRequest tokenResponse tokenData |

	code := aParameterDictionary at: 'code' ifAbsent: [ ^nil ].
	self authenticationState notNil
		ifTrue: [ self validateAuthenticationState: (aParameterDictionary at: 'state') ].

	tokenRequest  := (ZnRequest post: (ZnUrl fromString: self tokenEndpoint))
		entity: (ZnApplicationFormUrlEncodedEntity new
			at: 'code' put: code ;
			at: 'client_id' put: self consumerData key ;
			at: 'client_secret' put: self consumerData secret ;
			at: 'redirect_uri' put: self redirectUrl ;
			at: 'grant_type' put: 'authorization_code' ;
			yourself) ;
		yourself.

	tokenResponse := ZnClient new 
		request: tokenRequest ;
		post.
	tokenData := NeoJSONReader fromString: tokenResponse.
		
	self validateIdToken: (tokenData at: 'id_token').

	accessToken := tokenData at: 'access_token'.
	tokenType := tokenData at: 'token_type'.
	refreshToken := tokenData at: 'refresh_token' ifAbsent: [ nil ].! !

!ZnOpenIDConnectSession methodsFor: 'initialization-release' stamp: 'JanVanDeSandt 3/21/2014 13:22'!
initializeWith: aDiscoveryDocument consumerData: aConsumerData

	discoveryDocument := aDiscoveryDocument.
	self setConsumerData: aConsumerData ! !

!ZnOpenIDConnectSession methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/21/2014 13:55'!
issuer

	^discoveryDocument issuer! !

!ZnOpenIDConnectSession methodsFor: 'resources' stamp: 'JanVanDeSandt 3/21/2014 15:00'!
secureRandomString

	self error: 'Not Yet Implemented'! !

!ZnOpenIDConnectSession methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/21/2014 13:10'!
tokenEndpoint

	^discoveryDocument tokenEndpoint! !

!ZnOpenIDConnectSession methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/21/2014 13:34'!
userinfoEndpoint

	^discoveryDocument userinfoEndpoint! !

!ZnOpenIDConnectSession methodsFor: 'private' stamp: 'JanVanDeSandt 3/21/2014 14:51'!
validateAuthenticationState: aString

	self authenticationState = aString 
		ifFalse: [ self error: 'Invalid authentication state' ]! !

!ZnOpenIDConnectSession methodsFor: 'private' stamp: 'JanVanDeSandt 3/21/2014 14:43'!
validateIdToken: aString
	"TO DO!!"
	
	self error: 'Not Yet Implemented'! !

Object subclass: #ZnOpenIDConnectDiscoveryDocument
	instanceVariableNames: 'json'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-SSO-OAuth2-Core'!

!ZnOpenIDConnectDiscoveryDocument commentStamp: 'JanVanDeSandt 3/21/2014 15:07' prior: 0!
A ZnOpenIDConnectDiscoveryDocument is an object that contains discovered information about an OpenID Connect Provider.!

!ZnOpenIDConnectDiscoveryDocument class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 3/21/2014 11:32'!
fromUrl: anUrl

	^self new initializeWith: (
		NeoJSONReader fromString: (
			ZnClient new
				accept: ZnMimeType applicationJson ;
				get: anUrl))! !

!ZnOpenIDConnectDiscoveryDocument class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/21/2014 11:24'!
google

	^self fromUrl: self googleUrl ! !

!ZnOpenIDConnectDiscoveryDocument class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/21/2014 11:19'!
googleUrl

	^'https://accounts.google.com/.well-known/openid-configuration'! !

!ZnOpenIDConnectDiscoveryDocument methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/21/2014 11:25'!
authorizationEndpoint

	^json at: 'authorization_endpoint'! !

!ZnOpenIDConnectDiscoveryDocument methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 3/21/2014 11:28'!
initializeWith: aJsonDictionary

	json := aJsonDictionary! !

!ZnOpenIDConnectDiscoveryDocument methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/21/2014 11:25'!
issuer

	^json at: 'issuer'! !

!ZnOpenIDConnectDiscoveryDocument methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/21/2014 11:26'!
tokenEndpoint

	^json at: 'token_endpoint'! !

!ZnOpenIDConnectDiscoveryDocument methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/21/2014 11:26'!
userinfoEndpoint

	^json at: 'userinfo_endpoint'! !
